#************************************************************ 
#
# Component       CNF  
#
# Synopsis        Cluster Membership Monitor
#
# Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
#
#************************************************************
#
# #ident "@(#)CMM.pm 1.11 02/01/30 SMI"
#
#************************************************************

#=head1 Cluster configuration
#
#=over 4
#
#=cut

package CMM ;

use LOG ;


#
# variables to be exported
#
#
# information about nodes read into cluster file
#
@E_cluster_node = ();
$E_cluster_node_count = 0;
$E_local_node_id = -1 ;
$E_cluster_local_node = -1 ;

#
#  information about variables
#
#  WARNING: can be updated by command arguments at run time.
%E_variables = () ;
$E_cmm_configuration_file = "/etc/opt/SUNWcgha/nhcmm.conf" ;
$E_cmm_node_file = "/etc/opt/SUNWcgha/target.conf" ;
$E_cmm_cluster_file  = "/SUNWcgha/remote/etc/cluster_nodes_table" ;

my $G_cmmrole = "/opt/SUNWcgha/sbin/cmmrole" ;
my $G_not_configured = "/etc/opt/SUNWcgha/not_configured" ;
my $G_category = "CMM" ;

my $G_display_status = TRUE ;

#
#  valid values
#
my @G_valid_role = (
		    "MASTER" ,
		    "VICEMASTER" ,
		    "NONE"
		    ) ;


my @G_valid_parameter = (
			 "domain_id" ,
			 "attributes" ,
			 "role" ,
			 "election" ,
			) ;

#
#  information about nodes read into node-specific files
#
my %G_node_data ;




#==============================================================
#  used by other modules to get information
#==============================================================

#--------------------------------------------------------------
#
#  is_master_eligible
#
#    return TRUE if the local node is master-eligible
#
#--------------------------------------------------------------

sub is_master_eligible
  {
    if (($E_variables{CMM_IS_ELIGIBLE} eq "")
	or ($E_variables{CMM_IS_ELIGIBLE} eq "YES"))
      {
	return TRUE ;
      }

    return FALSE ;
  }

#--------------------------------------------------------------
#
#  get_actual_role
#
#    return the role by using cmmrole
#
#--------------------------------------------------------------

sub get_actual_role
  {
    $result = system("$G_cmmrole -to 5") ;
    $result = $result / 256 ;
    LOG::print_msg(2, "result of cmmrole: $result") ;
    
    if ($result eq 1)
      {
	return MASTER ;
      }
    if ($result eq 2)
      {
	return VICEMASTER ;
      }
    return UNKNOWN ;
  }

#--------------------------------------------------------------
#
#  get_node_id
#
#    return the local id of the node
#
#--------------------------------------------------------------

sub get_local_node_id
  {
    my $address ;
    my $nodeid ;

    # method extracted from the CGTP startup script
    $result = qx{netstat -in | egrep "${MAIN::E_subnet0}\.[0-9][0-9]*\.[12]\."} ;
    $address = (split(" ",$result))[3] ;
    if ($address eq "") {

#=item This node is not configured for a Netra HA Suite cluster (IP addresses not configured).
#
#    IP addresses are not compliant with the NHAS addressing schema.
#
#=cut

      LOG::print_error("This node is not configured for a Netra HA Suite cluster (IP addresses not configured)") ;
      exit 1;
    }
    $nodeid = (split('\.',$address))[3] ;

    return $nodeid ;
  }

#==============================================================
#  functions used for loading various files
#  handled by this module
#==============================================================

#--------------------------------------------------------------
#
#  load_cmm_configuration_file
#
#      load the CMM configuration file containing variable
#      definitiong
#
#--------------------------------------------------------------

sub load_cmm_configuration_file
  {

    LOG::print_msg(0, "Loading CMM configuration file $E_cmm_configuration_file ...") ;

    if (OS::check_file($E_cmm_configuration_file) eq FALSE)
	{
	  exit ;
	}

    $E_variables{CMM_NIC0} = "" ;
    $E_variables{CMM_NIC1} = "" ;
    $E_variables{CMM_DOMAIN_ID} = "" ;
    $E_variables{CMM_IS_ELIGIBLE} = "" ;

    open(FH, "<$E_cmm_configuration_file") ;
    while (<FH>)
      {
	chomp($line =$_) ;
	$initial = $line ;
	# remove comments
	$line =~ /([^\#]*)/;
	$line=$1 ;
	if ($line eq "")
	  {
	    next ;
	  }
	
	# retrieve parameter and its value
	if ($line =~ /[\s\t]*([^=\s\t]*)[\s\t]*=[\s\t]*([^=\s\t]*)/)
	  {
	    $parameter = $1 ;
	    $value = $2;
	    
	    LOG::print_msg(2, "set $parameter=$value");
	    $E_variables{$parameter} = $value;
	  }
      }
    close(FH) ;

  }
#--------------------------------------------------------------
#
#  load_node_specific_file
#
#      load the CMM node specific file
#
#--------------------------------------------------------------

sub load_node_specific_file
  {
    $L_status = OK ;
    
    LOG::print_msg(0,"Loading CMM node file $E_cmm_node_file ...") ;

    if (OS::check_file($E_cmm_node_file) eq FALSE)
	{
	  exit ;
	}

    LOG::print_msg(1,"loading $E_cmm_node_file") ;
    open(FH,"<$E_cmm_node_file") or die "$E_cmm_node_file: $!";
    
    while (<FH>)
      {
	chomp($line =$_) ;
	$initial = $line ;
	# remove comments
	$line =~ /([^\#]*)/;
	$line=$1 ;
	if ($line eq "")
	  {
	    next ;
	  }
	
	# retrieve parameter and its value
	$line =~ /[\s\t]*([^:\s\t]*)[\s\t]*:[\s\t]*([^:\s\t]*)/ ;
	$parameter = $1 ;
	$value = $2 ;

	if (($parameter eq "") || ($value eq ""))
	  {
	    # badly-formatted line : error

#=item ill-formatted line (<line number>)
#
#    The line contained in the file being processing is incorrect.
#    Please check its syntax.
#
#=cut

	    LOG::print_error("ill-formatted line ($.)");
	    $error = TRUE ;
	    next ;
	  }
	
	#
	#  check that the parameter name is valid
	#
	if (MAIN::isInArray(\@G_valid_parameter,scalar(@G_valid_parameter),
			    $parameter) ne TRUE)
	  {

#=item node <node name> <parameter>: invalid parameter
#
#    The specified parameter is not known.
#    Please check the syntax of the specified file.
#
#=cut

	    LOG::print_error("node $CMM::E_cluster_node{node_name}[$E_cluster_local_node] $parameter: invalid parameter") ;
	    $L_status = FAILED ;
	  }
	
	#
	#  check the role value is valid
	#
	if ($parameter eq "role")
	  {
	    if (MAIN::isInArray(\@G_valid_role,scalar(@G_valid_role),
				$value) ne TRUE)
	      {

#=item node <node name>: invalid role (<value>)
#
#    The specified role is not correct.
#
#=cut

		LOG::print_error("node $CMM::E_cluster_node{node_name}[$E_cluster_local_node]: invalid role ($value)") ;
		$L_status = FAILED ;
	      }
	  }
	
	$G_node_data{$parameter} = $value ;
      }
    close(FH) ;
    
    if (($G_display_status eq TRUE) || ($L_status ne OK))
      {
	LOG::print_result(
		     "node $CMM::E_cluster_node{node_name}[$E_cluster_local_node]: configuration file syntax",
		     $L_status);
      }
  }

#--------------------------------------------------------------
#
#  load_cluster_file
#
#--------------------------------------------------------------

sub load_cluster_file
  {
    LOG::print_msg(0,"Loading cluster node file $E_cmm_cluster_file ...") ;

    if (OS::check_file($E_cmm_cluster_file) eq FALSE)
	{
	  exit ;
	}

    open(FH,"<$E_cmm_cluster_file");

    while (<FH>)
      {
	chomp($line =$_) ;
	$initial = $line ;
	# remove comments
	$line =~ /([^\#]*)/;
	$line=$1 ;
	if ($line eq "")
	  {
	    next ;
	  }
	
	$line =~ m/[\s\t]*([0-9]*)[\s\t]*([0-9]*)[\s\t]*([^\s]*)[\s\t]*([^\s]*)/o ;
	LOG::print_msg(2, "node id=$1, domain_id=$2, node name=$3, attributes=$4");
	$E_cluster_node{node_id}[$E_cluster_node_count] = $1 ;
	$E_cluster_node{domain_id}[$E_cluster_node_count] = $2 ;
	$E_cluster_node{node_name}[$E_cluster_node_count] = $3 ;
	$E_cluster_node{attributes}[$E_cluster_node_count] = $4 ;

	if ($E_cluster_node{node_id}[$E_cluster_node_count] == $E_local_node_id)
	  {
	    $E_cluster_local_node = $E_cluster_node_count;
	  }
	$E_cluster_node_count++ ;
      }

    if ($E_cluster_local_node < 0)
      {

#=item local node (id=<node_id>) not defined in the cluster nodes table
#
#    The local node has no entry in the cluster nodes table.
#    Please add an entry for this node.
#
#=cut

	LOG::print_error("Local node (id=$E_local_node_id) not defined in the cluster nodes table") ;
	exit ;
      }

    close(FH)
  }


#--------------------------------------------------------------
#
#  check_local_node_specific_file
#
#      check the consistency of information of the local node
#      specific file
#
#--------------------------------------------------------------

sub check_local_node_specific_file
  {

    $G_display_success = TRUE ;

    #  check that all cluster data are identical for each node
    #  (be aware that subnet masks are not mandatory
    $L_status = OK ;
    if ($E_variables{CMM_DOMAIN_ID} ne $G_node_data{domain_id})
      {
	$L_status = FAILED ;
      }
    LOG::print_result("Domain id coherency in node specific file", $L_status) ;
  }


#--------------------------------------------------------------
#
#  check_cluster_file
#
#--------------------------------------------------------------

sub check_cluster_file
  {
    #
    # check that the domain id in the cluster file
    # is equal to the one in the cmm configuration file
    #
    $L_status = OK ;
    for ($i = 0 ; $i < $E_cluster_node_count ; $i++)
      {
	LOG::print_msg(2,"check domain_id CMM_DOMAIN_ID=$E_variables{CMM_DOMAIN_ID} node domain id=$E_cluster_node{domain_id}[$i]") ;
	if ($E_cluster_node{domain_id}[$i] != $E_variables{CMM_DOMAIN_ID})
	  {

#=item incorrect domain id for node <node name>
#
#    The domain id for the specified node is not equal to the value
#    indicated in the CMM configuration file (CMM_DOMAIN_ID variable).
#
#=cut

	    LOG::print_error("incorrect domain id for node $G_cluster_node{node_name}[$i]") ;
	    $L_status = FAILED ;
	  }
      }
    LOG::print_result("Domain ID coherency in cluster file",$L_status) ;

    #
    # check that node_id of each node is unique
    # and is not equal to nodes id reserved for master and vice master logical address
    #
    $L_status = OK ;
    for ($i = 0 ; $i < $E_cluster_node_count - 1 ; $i++)
      {

	if ($E_cluster_node{node_id}[$i] == $MAIN::E_master_node_id)
	  {

#=item Invalid node id for node <node>: <node id> reserved for master logical address
#
#    The specified node id is reserved for the master logical address.
#    It cannot be used as a node id for the specified node. Please change it.
#
#=cut

		LOG::print_error("Invalid node id for $E_cluster_node{node_name}[$i]: $E_cluster_node{node_id}[$i] reserved for master logical address ") ;
		$L_status = FAILED ;
	  }

	if ($E_cluster_node{node_id}[$i] == $MAIN::E_vicemaster_node_id)
	  {

#=item Invalid node id for node <node>: <node id> reserved for vice master logical address
#
#    The specified node id is reserved for the vice master logical address.
#    It can't be used as a node id for the specified node. Please change it.
#
#=cut

		LOG::print_error("Invalid node id for $E_cluster_node{node_name}[$i]: $E_cluster_node{node_id}[$i] reserved for vice master logical address ") ;
		$L_status = FAILED ;
	  }

	if ($E_cluster_node{node_id}[$i] == $MAIN::E_master_node_id)
	  {
	  }
	for ($j = $i+1 ; $j < $E_cluster_node_count ; $j++)
	  {
	    if ($E_cluster_node{node_id}[$i] == $E_cluster_node{node_id}[$j])
	      {

#=item <node1> and <node2> have the same node_id
#
#    Both nodes have the same node id in the cluster nodes table.
#    Please check that all nodes have a unique node id.
#
#=cut

		LOG::print_error("$E_cluster_node{node_name}[$i] and $E_cluster_node{node_name}[$j] have the same node_id") ;
		$L_status = FAILED ;
	      }
	  }
      }
    LOG::print_result("Node Id uniqueness in cluster file" ,$L_status) ;
  }

#--------------------------------------------------------------
#
#  check_not_configured
#
#  . check that configuration is available
#
#--------------------------------------------------------------

sub check_not_configured
  {
    $L_status = OK ;
    if (-f $G_not_configured)
      {

#=item Netra HA Suite services will not start
#
#    NHAS2 services will not started since the not_configured file exists.
#    Delete this file to enable the starting of services.
#
#=cut

	LOG::print_error("Netra HA Suite services will not start") ;
	$L_status = FAILED ;
      }
    LOG::print_result("File not_configured removed", $L_status); 
  }

#--------------------------------------------------------------
#
#  check_configuration
#
#      check the CMM configuration by:
#      . reading all the CMM node specific file
#        -> verify the consistency
#      . reading the CMM cluster file
#        -> verify the consistency
#
#--------------------------------------------------------------

sub check_configuration
  {
    LOG::print_msg(0,"") ;
    LOG::print_msg(0,"Node-specific file and  cluster file consistency check") ;
    LOG::print_msg(0,"") ;

    check_cluster_file ;
    if (is_master_eligible eq TRUE) {
      check_local_node_specific_file ;
    }

    check_not_configured ;

    #
    # check that /etc/services is correctly configured
    #
    OS::check_service("cmm-api","tcp") ;
    OS::check_service("cmm-config-push","udp") ;
    OS::check_service("cmm-config-pull","tcp") ;
    OS::check_service("cmm-election","udp") ;
    OS::check_service("cmm-cgtp-probe","udp") ;
    OS::check_service("cmm-cgtp-hb","udp") ;
  }

#--------------------------------------------------------------
#
#  load_all_files
#
#    load all files that are required for checking
#    (expect for installation)
#
#--------------------------------------------------------------

sub load_all_files
  {
    $E_local_node_id = get_local_node_id() ;

    load_cmm_configuration_file ;
    load_cluster_file ;
    if (is_master_eligible eq TRUE) {
      load_node_specific_file ;
    }
  }

#--------------------------------------------------------------
#
#  display
#
#--------------------------------------------------------------

sub display
  {
    $G_display_status = FALSE ;

    #
    #  Variables information
    #
    print "\n" ;
    print "CMM Variables\n\n" ;
    $first = TRUE ;
    foreach $var (%E_variables)
      {
	if ($first eq "TRUE")
	  {
	    $name = $var ;
	    $first = FALSE ;
	  }
	else
	  {
	    printf "    %-30s = %s\n", $name , $var ;
	    $first = TRUE ;
	  }
      }

    #
    #  cluster information
    #
    print "\n" ;
    print "CLuster information\n\n" ;
    for ($i = 0 ; $i < $E_cluster_node_count ; $i++)
      {
	print "    Node name:        $E_cluster_node{node_name}[$i]\n" ;
	print "    Node id:          $E_cluster_node{node_id}[$i]\n" ;
	print "    Domain id:        $E_cluster_node{domain_id}[$i]\n" ;
	print "    Attributes:       $E_cluster_node{attributes}[$i]\n" ;
	print "\n" ;
      }

    #
    #  node-specific information
    #

    if (is_master_eligible eq TRUE)
      {
	print "\n" ;
	print "Node-specific information\n\n" ;
	print "    Domain Id:        $G_node_data{domain_id}\n" ;
	print "    Role:             $G_node_data{role}\n" ;
	print "    Attributes:       $G_node_data{attributes}\n" ;
	print "    Election:         $G_node_data{election}\n" ;  
      }
  }

#=back
#
#=cut

{
}

